/***************************************************************************
                          s_partition_function.h  -  description
                             -------------------
    begin                : Wed Mar 15 2006
    copyright            : (C) 2006 by Mirela Andronescu
    email                : andrones@cs.ubc.ca
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PARTITION_FUNCTION_H
#define PARTITION_FUNCTION_H

//#include "s_stacked_pair.h"
#include "s_hairpin_loop.h"
#include "s_internal_loop.h"
#include "s_multi_loop.h"
#include "s_multi_loop_sub.h"


class s_partition_function
{
    public:

//        friend class s_stacked_pair;
//        friend class s_internal_loop;
//        friend class s_multi_loop;

        s_partition_function (char *seq);
        // The constructor

        ~s_partition_function ();
        // The destructor

        double compute_partition_function ();
  
        void compute_base_pair_probabilities ();
        // Nov 9, 2006. Computes base pair probabilities
        // PRE: the partition function arrays have been filled
        
        void PS_dot_plot(char *wastlfile);
        
        double *p;    // base pair probabilities
        
        double getOneoverRT () { return oneoverRT; }
        
        // exhaustive, for verification
        double compute_partition_function_exhaustively ();        
        double Z;
        double Zexhaustive;
        void verify_partition_function();
        
        void compute_logZ_gradient ();
        void compute_logZ_gradient_exhaustively ();
        void verify_gradient ();
              
    private:

        char *csequence;
        int *sequence;             // the entire sequence for which we compute the energy
        int seqlen;                 // sequence length
        int *index;                // an array with indexes, such that we don't work with a 2D array, but with a 1D array of length (n*(n+1))/2
        double oneoverRT;
        
        // arrays to compute the partition function
        
        // I'm using the terminology from Ding & Lawrence NAR 2003, "A statistical sampling algorithm for RNA secondary structure prediction"
        double *u;     // partition function on region i,j (can have anything)
        double *up;    // (i,j) paired
        double *s1;    // helper
        
        // arrays for multi-loops
        double *upm;    // (i,j) closes a multi-loop
        double *s2;    // helper
        double *u1;    // partial multi-loop
        double *s3;    // helper
        
        // arrays for base pair probabilities        
        double *pm;
        double *pm1;
        
        void initialize_arrays ();
        void compute_u (int i, int j);
        void compute_up (int i, int j);
        void compute_upm (int i, int j);
        void compute_u1 (int i, int j);
        void compute_s1 (int h, int j);
        void compute_s2 (int h, int j);        
        void compute_s3 (int h, int j);
        double exp_AUpenalty (int i, int j);
        double exp_dangle5 (int i, int j, int k);
        double exp_dangle3 (int i, int j, int k);        
        double f (int jplus1, int h, int l);
        void compute_p (int h, int l);
        void compute_pm (int h, int l);
        void compute_pm1 (int h, int l);
        
        double *pexhaustive;
        double *upexhaustive;
        int has_base_pair (int i, int j, char *structure);
        int identical_structure (int i, int j, char *structure1, char *structure2);
        double exp_free_energy_partial (int i, int j, char *sequence, char *structure);
        
        double *GlogZ;    // gradient of logZ
        double *GlogZexhaustive;    
};



#endif


